import os
from generation_models import message_construct_func, GPT_response
import re
import pandas as pd
import subprocess
from argparse import ArgumentParser

def extract_code(text):
    # Regular expression to match code blocks enclosed in triple backticks
    code_block_pattern = re.compile(r'```python\n(.*?)\n```', re.DOTALL)

    # Find all matches in the text
    code_blocks = code_block_pattern.findall(text)

    # If no code blocks are found, try to find indented code blocks
    if not code_blocks:
        return []

    return code_blocks

def extract_equation_with_GPT4(response):
    prompt = 'Your task is to extract the equation from the given answer by another LLM:\n' \
             'Note that the equation should include four numbers be in the form like ((11 * 8) + 8) / 4, ((3 * 5) - 12) * 8, ((7 - 4) * 11) - 9 = 24,' \
             '((6 / 3) * 7) + 10, (37 - (29 - 16)), ((19 + 18) - 13) = 24\n' \
             'Here is the reponse, return your answer with the format <<<equation>>>, like <<<((7 - 4) * 11) - 9 = 24>>>. ' \
             'Input text: ' \

    extract_equation = GPT_response('', prompt + response, model_name='gpt-4o', code_interpreter=False, user_prompt_list=[prompt + response], response_total_list=[])
    return extract_equation

def extract_and_check(response):
    # Extract all texts between <<< and >>>
    matches = re.findall(r'<<<(.*?)>>>', response)
    extracted_text = matches[-1].strip() if matches else ''

    # Check if 'itertools' appears in the response
    itertools_present = 'code_interpreter' in response or '```python' in response

    return extracted_text, itertools_present

def validate_equation(number_list, extracted_text):
    # Check if extracted_text has both left and right side
    print(f'extracted_text: {extracted_text}')
    print(extracted_text.split("="))

    if "=" not in extracted_text:
        extracted_text = extracted_text + " = 24"

    # Split the extracted_text into left and right parts
    equation_part_list = extracted_text.split("=")
    if len(equation_part_list) == 2:
        left_side = equation_part_list[0]
        right_side = equation_part_list[1]
    elif len(equation_part_list) > 2:
        left_side = equation_part_list[0]
        right_side = equation_part_list[-1]
    #left_side, right_side = extracted_text.split("=")

    # Evaluate the right side
    try:
        right_value = eval(right_side.strip())
        if right_value != 24:
            return False
    except:
        return False

    # Prepare the number list as a multiset
    number_multiset = sorted(number_list)

    # Extract numbers and operators from the left side using regex
    # Supports LaTeX expressions as well
    left_side_numbers = re.findall(r'\d+', left_side)
    left_side_numbers = list(map(int, left_side_numbers))
    left_side_numbers_sorted = sorted(left_side_numbers)

    if number_multiset != left_side_numbers_sorted:
        return False

    # Replace LaTeX syntax with Python syntax for evaluation
    left_side = left_side.replace(r'\times', '*').replace(r'\div', '/')
    left_side = re.sub(r'\\frac{(\d+)}{(\d+)}', r'(\1/\2)', left_side)

    # Evaluate the left side
    try:
        left_value = eval(left_side.strip())
        if left_value != 24:
            return False
    except:
        return False

    return True

#post_process_game24
def post_process_game24(dataset_input_dir, save_input_dir, target_value, value_num, model_name, AutoGen_prompt_system_message,
               AutoGen_prompt_concatenate, code_interpreter, Encourage_code_execution_interpreter, all_code_with_COT,
               all_code_without_COT, all_text, multi_turn_planning, multi_turn_planning_round_num, method_9_all_text_all_code_summarizer, method_10_LLM_estimates_scores_first):
    print('\n' + '*'*30)
    print(f'Target_value: {target_value}, Value_num: {value_num}, model_name: {model_name}, AutoGen_prompt_system_message: {AutoGen_prompt_system_message}, '
          f'AutoGen_prompt_concatenate: {AutoGen_prompt_concatenate}, code_interpreter: {code_interpreter}, Encourage_code_execution_interpreter: {Encourage_code_execution_interpreter}, '
          f'all_code_with_COT: {all_code_with_COT}, all_code_without_COT: {all_code_without_COT}, all_text: {all_text}, multi_turn_planning: {multi_turn_planning}, multi_turn_planning_round_num: {multi_turn_planning_round_num}\n')

    dataset_base_dir = dataset_input_dir + f'/data/{target_value}'

    if target_value == 24 and value_num == 4:
        dataset_csv = dataset_base_dir + f'/24.csv'
    else:
        dataset_csv = dataset_bas

        e_dir + f'/{target_value}_{value_num}_number.csv'
    df = pd.read_csv(dataset_csv)

    total_sample_num = 0
    answer_with_code_wrong = 0
    answer_with_code_correct = 0
    answer_without_code_wrong = 0
    answer_without_code_correct = 0

    if target_value == 24 and value_num == 4:
        if method_9_all_text_all_code_summarizer == False and method_10_LLM_estimates_scores_first == False:
            base_save_code_dir = save_input_dir + f'/result_game24_{model_name}_interpreter_{code_interpreter}_AutoGen_system_{AutoGen_prompt_system_message}_AutoGen_concatenate_{AutoGen_prompt_concatenate}_all_code_with_COT_{all_code_with_COT}_all_code_without_COT_{all_code_without_COT}_Encourage_code_execution_interpreter_{Encourage_code_execution_interpreter}_all_text_{all_text}_multi_turn_planning_{multi_turn_planning}_round_num_{multi_turn_planning_round_num}'
        elif method_9_all_text_all_code_summarizer == True and method_10_LLM_estimates_scores_first == False:
            base_save_code_dir = save_input_dir + f'/result_game24_{model_name}_method_9_all_text_all_code_summarizer'
        elif method_9_all_text_all_code_summarizer == False and method_10_LLM_estimates_scores_first == True:
            base_save_code_dir = save_input_dir + f'/result_game24_{model_name}_method_10_LLM_estimates_scores_first'
        else:
            raise ValueError(
                "method_9_all_text_all_code_summarizer and method_10_LLM_estimates_scores_first can't be both True")
    else:
        if method_9_all_text_all_code_summarizer == False and method_10_LLM_estimates_scores_first == False:
            base_save_code_dir = save_input_dir + f'/result_game{target_value}_{value_num}_number_{model_name}_interpreter_{code_interpreter}_AutoGen_system_{AutoGen_prompt_system_message}_AutoGen_concatenate_{AutoGen_prompt_concatenate}_all_code_with_COT_{all_code_with_COT}_all_code_without_COT_{all_code_without_COT}_Encourage_code_execution_interpreter_{Encourage_code_execution_interpreter}_all_text_{all_text}_multi_turn_planning_{multi_turn_planning}_round_num_{multi_turn_planning_round_num}'
        elif method_9_all_text_all_code_summarizer == True and method_10_LLM_estimates_scores_first == False:
            base_save_code_dir = save_input_dir + f'/result_game{target_value}_{value_num}_number_{model_name}_method_9_all_text_all_code_summarizer'
        elif method_9_all_text_all_code_summarizer == False and method_10_LLM_estimates_scores_first == True:
            base_save_code_dir = save_input_dir + f'/result_game{target_value}_{value_num}_number_{model_name}_method_10_LLM_estimates_scores_first'

    if multi_turn_planning:
        round_number = multi_turn_planning_round_num
    else:
        round_number = 1

    for j in range(0, 1):
        for i in range(j, min(len(df), 1500), 10):
            total_sample_num += 1
            number_list = list(map(int, df['Puzzles'][i].split()))
            save_code_dir = os.path.join(base_save_code_dir, f'sample_{i}')
            #with open(save_code_dir + "/input_prompt.txt", "r") as f:
            #    input_prompt = f.read()
            with open(save_code_dir + "/response_code_1.txt", "r") as f:
                response = f.read()
            print(f'\nSample num: {i}\n')
            print(
                f'\nTarget_value: {target_value}, Value_num: {value_num}, model_name: {model_name}, AutoGen_prompt_system_message: {AutoGen_prompt_system_message}, AutoGen_prompt_concatenate: {AutoGen_prompt_concatenate}, code_interpreter: {code_interpreter}, Encourage_code_execution_interpreter: {Encourage_code_execution_interpreter}, all_code_with_COT: {all_code_with_COT}, all_code_without_COT: {all_code_without_COT}, all_text: {all_text}')

            code_block_list = extract_code(response)
            for index, code_string in enumerate(code_block_list):
                with open(save_code_dir + f"/code_1_{index}.py", "w") as f:
                    f.write(code_string)
                #print(f'code_{index}:\n {code_string}')

            # Test the generated code
            if not os.path.exists(save_code_dir + f"/code_1_0.py"):
                extracted_text, itertools_present = extract_and_check(response)
            else:
                try:
                    result = subprocess.run(
                        ["python3", "-c", f"exec(open('{save_code_dir}/code_1_0.py').read()); print(result)"],
                        capture_output=True,
                        text=True,
                        timeout=15
                    )
                    '''
                    result = subprocess.run(
                        ["python3", save_code_dir + f"/code_1_0.py"],
                        capture_output=True, text=True, timeout=15)
                    '''
                    output = result.stdout
                    errors = result.stderr
                except Exception as e:
                    output = ""
                    errors = str(e)
                extracted_text, itertools_present = extract_and_check(output)
                itertools_present = True
            if extracted_text == "" or len(extracted_text) < 5:
                output = None
                iteration_num_1 = 0
                while output == None and iteration_num_1 < 3:
                    iteration_num_1 += 1
                    output = extract_equation_with_GPT4(response)
                if output == None:
                    output = ''
                extracted_text, _ = extract_and_check(output)

            if validate_equation(number_list, extracted_text) == False:
                print('False')
                # print(f"response: {response}\n")
                if itertools_present == True:
                    answer_with_code_wrong += 1
                    print(f'\nnumber_list: {number_list}')
                    print(f"response: {response}\n")
                elif itertools_present == False:
                    answer_without_code_wrong += 1
                #print(f"response: {response}\n")
                print(f'\nnumber_list: {number_list}')
                print(f'extracted_text: {extracted_text}')
                #print(f'itertools_present: {itertools_present}\n')
                if extracted_text == "":
                    print(f"response: {response}\n")

            elif validate_equation(number_list, extracted_text) == True:
                print('True')
                if itertools_present == True:
                    answer_with_code_correct += 1
                elif itertools_present == False:
                    answer_without_code_correct += 1

            with open(save_code_dir + f"/response_answer.txt", "w") as f:
                f.write(extracted_text)
            with open(save_code_dir + f"/success_failure.txt", "w") as f:
                f.write(str(validate_equation(number_list, extracted_text)))

    print(f'\ntotal_sample_num: {total_sample_num}')
    print(f'answer_with_code_wrong: {answer_with_code_wrong}')
    print(f'answer_without_code_wrong: {answer_without_code_wrong}')
    print(f'answer_with_code_correct: {answer_with_code_correct}')
    print(f'answer_without_code_correct: {answer_without_code_correct}')

    with open(base_save_code_dir + f"/answer_with_code_wrong.txt", "w") as f:
        f.write(str(answer_with_code_wrong))
    with open(base_save_code_dir + f"/answer_without_code_wrong.txt", "w") as f:
        f.write(str(answer_without_code_wrong))
    with open(base_save_code_dir + f"/answer_with_code_correct.txt", "w") as f:
        f.write(str(answer_with_code_correct))
    with open(base_save_code_dir + f"/answer_without_code_correct.txt", "w") as f:
        f.write(str(answer_without_code_correct))

    print(f'\nTarget_value: {target_value}, Value_num: {value_num}, model_name: {model_name}, AutoGen_prompt_system_message: {AutoGen_prompt_system_message}, AutoGen_prompt_concatenate: {AutoGen_prompt_concatenate}, code_interpreter: {code_interpreter}, Encourage_code_execution_interpreter: {Encourage_code_execution_interpreter}, all_code_with_COT: {all_code_with_COT}, all_code_without_COT: {all_code_without_COT}, all_text: {all_text}')
    print('*' * 30)

    run_info = f"{model_name}, round_num={multi_turn_planning_round_num}, " \
               f"{code_interpreter}, {AutoGen_prompt_system_message}, {AutoGen_prompt_concatenate}, " \
               f"{Encourage_code_execution_interpreter}, {all_code_with_COT}, {all_code_without_COT}, {all_text}, {multi_turn_planning}, multi_turn_planning_round_num: {multi_turn_planning_round_num}" \
               f"\nmethod_9_{method_9_all_text_all_code_summarizer}, method_10_{method_10_LLM_estimates_scores_first}\n"

    run_info_result = f'answer_with_code_wrong:{answer_with_code_wrong}, answer_without_code_wrong: {answer_without_code_wrong}, answer_with_code_correct: {answer_with_code_correct}, answer_without_code_correct: {answer_without_code_correct}\n'
    log_file_result = os.path.join(save_input_dir, f"acc_result_log_{model_name}.txt")
    log_run_info(log_file_result, run_info + run_info_result)

if __name__ == '__main__':
    # gpt-4o, gpt-4o-mini, gpt-3.5-turbo for OpenAi API
    # gpt-4o, gpt-35-turbo for Azure API
    # multi_turn_planning_round_num = [1, 2, 4, 8, 16]

    parser = ArgumentParser()
    parser.add_argument('-model_name', '--model_name', default='gpt-35-turbo-16k-0613')
    args = parser.parse_args()
    model_name = args.model_name

    dataset_input_dir = '../dataset_gather'
    save_input_dir = '../results_gather/game24'

    if not os.path.exists(save_input_dir):
        os.makedirs(save_input_dir)

    def log_run_info(log_file, run_info):
        with open(log_file, 'a') as f:
            f.write(run_info + "\n")

    #for multi_turn_planning_round_num in [1, 2, 4, 8, 16]:
    for multi_turn_planning_round_num in [1]:
        #for model_name in ['gpt-4o', 'gpt-4o-mini', 'gpt-3.5-turbo', 'gpt-35-turbo-16k-0613']:
        #for model_name in ['gpt-4o', 'gpt-4o-mini', 'gpt-3.5-turbo', 'gpt-35-turbo-16k-0613']:
        log_file = os.path.join(save_input_dir, f"post_process_log_{model_name}.txt")

        for dataset_input_dir, save_input_dir, target_value, value_num, model_name, code_interpreter, AutoGen_prompt_system_message, AutoGen_prompt_concatenate,\
            Encourage_code_execution_interpreter, all_code_with_COT, all_code_without_COT, all_text, multi_turn_planning, multi_turn_planning_round_num\
                , method_9_all_text_all_code_summarizer, method_10_LLM_estimates_scores_first in [
            #(dataset_input_dir, save_input_dir, 24, 4, model_name, False, False, False, False, False, False, False, True, multi_turn_planning_round_num, False, False),
            #(dataset_input_dir, save_input_dir, 24, 4, model_name, False, True, False, False, False, False, False, True, multi_turn_planning_round_num, False, False),
            #(dataset_input_dir, save_input_dir, 24, 4, model_name, False, False, True, False, False, False, False, True, multi_turn_planning_round_num, False, False),
            #(dataset_input_dir, save_input_dir, 24, 4, model_name, False, False, False, False, False, False, True, True, multi_turn_planning_round_num, False, False),
            #(dataset_input_dir, save_input_dir, 24, 4, model_name, False, False, False, False, True, False, False, True, multi_turn_planning_round_num, False, False),
            #(dataset_input_dir, save_input_dir, 24, 4, model_name, False, False, False, False, False, True, False, True, multi_turn_planning_round_num, False, False),
            (dataset_input_dir, save_input_dir, 24, 4, model_name, False, False, False, False, False, False, False, True, multi_turn_planning_round_num, True, False),
            (dataset_input_dir, save_input_dir, 24, 4, model_name, False, False, False, False, False, False, False, True, multi_turn_planning_round_num, False, True),
            ]:

            post_process_game24(dataset_input_dir, save_input_dir, target_value, value_num, model_name, AutoGen_prompt_system_message,
                                AutoGen_prompt_concatenate, code_interpreter, Encourage_code_execution_interpreter, all_code_with_COT,
                                all_code_without_COT, all_text, multi_turn_planning, multi_turn_planning_round_num, method_9_all_text_all_code_summarizer, method_10_LLM_estimates_scores_first)

            # Log the completed run
            run_info = f"Completed run: {model_name}, round_num={multi_turn_planning_round_num}, " \
                       f"{code_interpreter}, {AutoGen_prompt_system_message}, {AutoGen_prompt_concatenate}, " \
                       f"{Encourage_code_execution_interpreter}, {all_code_with_COT}, {all_code_without_COT}, {all_text}, {multi_turn_planning}, multi_turn_planning_round_num: {multi_turn_planning_round_num}" \
                       f"\nmethod_9_{method_9_all_text_all_code_summarizer}, method_10_{method_10_LLM_estimates_scores_first}\n"
            log_run_info(log_file, run_info)


'''
    #for model_name in ['gpt-4o', 'gpt-4o-mini', 'gpt-3.5-turbo', 'gpt-35-turbo-16k-0613']:
    #for model_name in ['gpt-4o', 'gpt-4o-mini', 'gpt-3.5-turbo', 'gpt-35-turbo-16k-0613']:
    log_file = os.path.join(save_input_dir, f"post_process_log_{model_name}.txt")

    for dataset_input_dir, save_input_dir, target_value, value_num, model_name, code_interpreter, AutoGen_prompt_system_message, AutoGen_prompt_concatenate,\
        Encourage_code_execution_interpreter, all_code_with_COT, all_code_without_COT, all_text, multi_turn_planning, multi_turn_planning_round_num
         , method_9_all_text_all_code_summarizer, method_10_LLM_estimates_scores_first in [
        (dataset_input_dir, save_input_dir, 24, 4, model_name, True, False, False, False, False, False, False, False, 1),
        (dataset_input_dir, save_input_dir, 24, 4, model_name, True, False, False, True, False, False, False, False, 1)]:

        post_process_game24(dataset_input_dir, save_input_dir, target_value, value_num, model_name, AutoGen_prompt_system_message,
                            AutoGen_prompt_concatenate, code_interpreter, Encourage_code_execution_interpreter, all_code_with_COT,
                            all_code_without_COT, all_text, multi_turn_planning, multi_turn_planning_round_num, method_9_all_text_all_code_summarizer, method_10_LLM_estimates_scores_first)

        # Log the completed run
        run_info = f"Completed run: {model_name}, round_num={multi_turn_planning_round_num}, " \
                   f"{code_interpreter}, {AutoGen_prompt_system_message}, {AutoGen_prompt_concatenate}, " \
                   f"{Encourage_code_execution_interpreter}, {all_code_with_COT}, {all_code_without_COT}, {all_text}, {multi_turn_planning}"
        log_run_info(log_file, run_info)
'''

'''
if __name__ == '__main__':
    # gpt-4o, gpt-4o-mini, gpt-3.5-turbo for OpenAi API
    # gpt-4o, gpt-35-turbo for Azure API
    model_name = 'gpt-3.5-turbo'
    for target_value, value_num, model_name, code_interpreter, AutoGen_prompt_system_message, AutoGen_prompt_concatenate,\
        Encourage_code_execution_interpreter, all_code_with_COT, all_code_without_COT, all_text in [
        (24, 5, model_name, False, False, False, False, False, False, True), (24, 5, model_name, False, False, False, False, False, True, False),
        (24, 3, model_name, False, False, False, False, False, False, True), (24, 3, model_name, False, False, False, False, False, True, False),
        (24, 2, model_name, False, False, False, False, False, False, True), (24, 2, model_name, False, False, False, False, False, True, False)]:
        post_process_game24(target_value, value_num, model_name, AutoGen_prompt_system_message, AutoGen_prompt_concatenate,
                   code_interpreter, Encourage_code_execution_interpreter, all_code_with_COT, all_code_without_COT, all_text)
'''